/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * This class provide methods
 * to access device database
 *
 * @author Fabio Maggi @ Funambol
 * $Id: DataStore.java,v 1.2 2004/05/10 10:50:23 fabius Exp $
 */

package sync4j.syncclient.sps;

import java.util.Vector;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordEnumeration;

import sync4j.syncclient.util.StaticDataHelper;


public class DataStore {

    //------------------------------------------------------------- Constants

    private static final String TIMESTAMP_RECORDSTORE   = "TimeStampRS";

    private static final char   RECORD_STATE_NEW      = 'N'            ;
    private static final char   RECORD_STATE_DELETED  = 'D'            ;
    private static final char   RECORD_STATE_UPDATED  = 'U'            ;
    private static final char   RECORD_STATE_UNSIGNED = ' '            ;

    //------------------------------------------------------------- Private Data

    private String recordStoreName = null;

    //------------------------------------------------------------- Costructors

    public DataStore (String recordStoreName) {
        this.recordStoreName = recordStoreName;
    }

    //------------------------------------------------------------- Public methods

    /**
     * Set last timestamp in dedicate recordStore
     * @param lastTimestamp
     * @throws DataAccessException
     **/
    public void setLastTimestamp(long lastTimestamp)
    throws DataAccessException {

        RecordStore recordStore = null;

        try {

            recordStore = RecordStore.openRecordStore(TIMESTAMP_RECORDSTORE, true);

            int numRecords = recordStore.getNumRecords();

            String recordValue = String.valueOf(lastTimestamp);

            if (numRecords == 1) {
                recordStore.setRecord(1, recordValue.getBytes(), 0, (recordValue.getBytes()).length);
            } else {
                recordStore.addRecord(recordValue.getBytes(), 0, (recordValue.getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

    }


    /**
     * @return last timestamp from dedicate recordstore
     * @throws DataAccessException
     **/
    public long getLastTimestamp()
    throws DataAccessException {

        RecordStore recordStore = null;

        long lastTimestamp = 0;

        try {

            recordStore = RecordStore.openRecordStore(TIMESTAMP_RECORDSTORE, true);

            int numRecords = recordStore.getNumRecords();

            if (numRecords == 0) {
                lastTimestamp = 0;
            } else {
                lastTimestamp = Long.parseLong(new String(recordStore.getRecord(1)));
            }

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

        return lastTimestamp;

    }


    /**
     * if record exist in database, update records
     * if record not exist in database, add record
     * @param record record to store
     * @throws DataAccessException
     **/
    public Record setRecord(Record record)
    throws DataAccessException {

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(record.getKey() + "|");

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(recordStoreName, true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {

                idRecord = recordEnumeration.nextRecordId();

            }

            String recordValue = record.toString();

            if (idRecord != -1) {
                recordStore.setRecord(idRecord, recordValue.getBytes(), 0, (recordValue.getBytes()).length);
            } else {
                recordStore.addRecord(recordValue.getBytes(), 0, (recordValue.getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

        return record;

    }


    /**
     * delete record
     * @param record
     * @throws DataAccessException
     **/
    public void deleteRecord(Record record)
    throws DataAccessException {

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(record.getKey() + "|");

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(recordStoreName, true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {

                idRecord = recordEnumeration.nextRecordId();

            }

            if (idRecord == -1) {
                //errore record non esistente
                return;
            } else {

                recordStore.deleteRecord(idRecord);

            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

    }


    /**
     * return no deleted records from device recordstore
     *
     * @return find records
     * @throws DataAccessException
     **/
    public Vector getNoDeletedRecords()
    throws DataAccessException {


        Vector newRecords      = null ;
        Vector updateRecords   = null ;
        Vector unsignedRecords = null ;

        Vector records         = null ;

        newRecords      = getRecords(RECORD_STATE_NEW      );
        updateRecords   = getRecords(RECORD_STATE_UPDATED  );
        unsignedRecords = getRecords(RECORD_STATE_UNSIGNED );

        records = newRecords;

        for (int i=0, l = updateRecords.size(); i < l; i++) {
            records.addElement(updateRecords.elementAt(i));
        }

        for (int i=0, l = unsignedRecords.size(); i < l; i++) {
            records.addElement(unsignedRecords.elementAt(i));
        }

        return records;

    }


    /**
     * return record from recordstore
     * filter by record state
     *
     * @return find records
     * @throws DataAccessException
     **/
    public Vector getRecords(char state)
    throws DataAccessException {

        Vector records = new Vector();

        RecordStore recordStore = null;

        SimpleComparator simpleComparator = null;
        StateFilterRecords stateFilterRecords = null;

        RecordEnumeration recordEnumeration = null;

        Record record = null;

        String key = null;

        try {

            recordStore = RecordStore.openRecordStore(recordStoreName, true);

            simpleComparator = new SimpleComparator();

            stateFilterRecords = new StateFilterRecords(String.valueOf(state));

            recordEnumeration = recordStore.enumerateRecords(stateFilterRecords, simpleComparator, false);

            while (recordEnumeration.hasNextElement()) {

                record = new Record(new String(recordEnumeration.nextRecord()));

                records.addElement(record);

            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

        return records;

    }

    /**
     * execute init recordstore operations
     */
    public void startDSOperations() {

    }

    /**
     * execute commit recordstore operations
     * remove records signed as DELETED 'D'
     * mark UNSIGNED ' ' records signed as NEW 'N' and UPDATED 'U'
     *
     * @throws DataAccessException
     *
     */
    public void commitDSOperations()
    throws DataAccessException {

        Record record = null;

        RecordStore recordStore = null;

        StateFilterRecords stateFilterRecords = null;

        RecordEnumeration recordEnumeration = null;

        try {

            recordStore = RecordStore.openRecordStore(recordStoreName, true);

            //
            // remove records signed as DELETED 'D'
            //
            stateFilterRecords = new StateFilterRecords(String.valueOf(RECORD_STATE_DELETED));
            recordEnumeration = recordStore.enumerateRecords(stateFilterRecords, null, false);
            while (recordEnumeration.hasNextElement()) {
                recordStore.deleteRecord(recordEnumeration.nextRecordId());
            }

            //
            // mark UNSIGNED ' ' records signed as NEW 'N'
            //
            stateFilterRecords = new StateFilterRecords(String.valueOf(RECORD_STATE_NEW));
            recordEnumeration = recordStore.enumerateRecords(stateFilterRecords, null, false);
            while (recordEnumeration.hasNextElement()) {
                int id = recordEnumeration.nextRecordId();
                record = new Record(new String(recordStore.getRecord(id)));
                record.setState(RECORD_STATE_UNSIGNED);
                recordStore.setRecord(id, record.toString().getBytes(), 0, (record.toString().getBytes()).length);
            }

            //
            // mark UNSIGNED ' ' records signed as UPDATED 'U'
            //
            stateFilterRecords = new StateFilterRecords(String.valueOf(RECORD_STATE_UPDATED));
            recordEnumeration = recordStore.enumerateRecords(stateFilterRecords, null, false);
            while (recordEnumeration.hasNextElement()) {
                int id = recordEnumeration.nextRecordId();
                record = new Record(new String(recordStore.getRecord(id)));
                record.setState(RECORD_STATE_UNSIGNED);
                recordStore.setRecord(id, record.toString().getBytes(), 0, (record.toString().getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            throw new DataAccessException(e.getMessage());
        }

    }


    public long getNextKey() {
        //da implementare
        return 0;
    }

}