/**
 * Copyright (C) 2003-2004 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.syncclient.spds;

import java.util.Hashtable;

/**
 * This is a wrapper to create a istance of SyncManagerImpl
 *
 * @version  $Id: SyncManager.java,v 1.2 2004/06/16 09:51:42 fabius Exp $
 * @author Fabio Maggi
 */
public class SyncManager {

    //---------------------------------------------------------------- Constants

    private static final String KEY_LOGIN       = "login"      ;
    private static final String KEY_SERVER_URL  = "serverUrl"  ;

    private static final String KEY_CHARSET     = "charset"    ;

    //---------------------------------------------------------------- Private data

    SyncManagerImpl syncManagerImpl = null;

    //---------------------------------------------------------------- Constructors

    /**
     * Creates a SyncManager.
     * The datastore is identified by the dataStoreName parameter
     *
     * @param dataStoreName The dataStore to Sync
     *
     */
    public SyncManager (String dataStoreName, Hashtable properties) {

        this.syncManagerImpl = new SyncManagerImpl(dataStoreName);

        if (properties.get(KEY_LOGIN) != null) {
            this.syncManagerImpl.setLogin ((String) properties.get(KEY_LOGIN));
        }

        if (properties.get(KEY_SERVER_URL) != null) {
            this.syncManagerImpl.setServerUrl ((String) properties.get(KEY_SERVER_URL));
        }

        if (properties.get(KEY_CHARSET) != null) {
            this.syncManagerImpl.setCharset ((String) properties.get(KEY_CHARSET));
        }

    }

    //----------------------------------------------------------- Public methods
    /**
     * Synchronize data.
     * @throws SyncException if an error occurs during synchronization
     * @throws AuthenticationException if the server responded with "non authorized" return code
     */
    public void sync() throws SyncException, AuthenticationException, UpdateException {

        this.syncManagerImpl.sync();

    }

}